/****************************************************************************
*                                                                           *
*  OpenNI 1.x Alpha                                                         *
*  Copyright (C) 2011 PrimeSense Ltd.                                       *
*                                                                           *
*  This file is part of OpenNI.                                             *
*                                                                           *
*  OpenNI is free software: you can redistribute it and/or modify           *
*  it under the terms of the GNU Lesser General Public License as published *
*  by the Free Software Foundation, either version 3 of the License, or     *
*  (at your option) any later version.                                      *
*                                                                           *
*  OpenNI is distributed in the hope that it will be useful,                *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of           *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             *
*  GNU Lesser General Public License for more details.                      *
*                                                                           *
*  You should have received a copy of the GNU Lesser General Public License *
*  along with OpenNI. If not, see <http://www.gnu.org/licenses/>.           *
*                                                                           *
****************************************************************************/
//---------------------------------------------------------------------------
// Includes
//---------------------------------------------------------------------------
#include "TrackingInitializer.h"

TrackingInitializer::TrackingInitializer(xn::UserGenerator *pUserGenerato) : 
                                                       m_pUserSelector(NULL),
                                                       m_pUserGenerator(pUserGenerato),
                                                       m_bValid(FALSE)
{
    if(m_pUserGenerator==NULL)
        return; // nothing to work with so we stay invalid.
    m_bValid=TRUE;
}

TrackingInitializer::~TrackingInitializer()
{

}

XnStatus TrackingInitializer::SetUserTracker(UserSelector *pUserSelector)
{
    if(m_pUserSelector!=NULL)
        return XN_STATUS_ERROR; // we already have a user selector
    m_pUserSelector=pUserSelector;
    return XN_STATUS_OK;
}

