@%0\..\..\..\BuildJava.py "%1" "%0\.." ..\..\..\..\..\Samples\SimpleViewer.java\org\OpenNI\Samples\SimpleViewer org.OpenNI.Samples.SimpleViewer org.OpenNI.jar org.OpenNI.Samples.SimpleViewer.SimpleViewerApplication
