#include <windows.h>
#include <ntverp.h>
#include "PSDrvVersion.h"

#define VER_FILETYPE                VFT_DRV
#define VER_FILESUBTYPE             VFT2_DRV_SYSTEM
#define VER_FILEDESCRIPTION_STR     "Prime Sensor Device Driver v3.0"
#define VER_PRODUCTNAME_STR         "Prime Sensor Device Driver v3.0"
#define VER_INTERNALNAME_STR        "psdrv3.sys"
#define VER_ORIGINALFILENAME_STR    "psdrv3.sys"
#define VER_FILEVERSION             PSUSBDRV_MAJOR_VERSION,PSUSBDRV_MINOR_VERSION,PSUSBDRV_MAINTENANCE_VERSION,PSUSBDRV_BUILD_VERSION
#define VER_FILEVERSION_STR         PSDRV_BRIEF_VERSION_STRING
#define VER_PRODUCTVERSION_STR      PSDRV_BRIEF_VERSION_STRING
#define VER_COMPANYNAME_STR         "Prime Sense Ltd."
#define VER_LEGALCOPYRIGHT_STR      "(c) Prime Sense Ltd. All rights reserved."
#define __BUILDMACHINE__			WinDDK-7600.16385.0

#include "common.ver"
