/****************************************************************************
*                                                                           *
*  OpenNI 1.x Alpha                                                         *
*  Copyright (C) 2011 PrimeSense Ltd.                                       *
*                                                                           *
*  This file is part of OpenNI.                                             *
*                                                                           *
*  OpenNI is free software: you can redistribute it and/or modify           *
*  it under the terms of the GNU Lesser General Public License as published *
*  by the Free Software Foundation, either version 3 of the License, or     *
*  (at your option) any later version.                                      *
*                                                                           *
*  OpenNI is distributed in the hope that it will be useful,                *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of           *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             *
*  GNU Lesser General Public License for more details.                      *
*                                                                           *
*  You should have received a copy of the GNU Lesser General Public License *
*  along with OpenNI. If not, see <http://www.gnu.org/licenses/>.           *
*                                                                           *
****************************************************************************/
#ifndef __NICODECIDS_H__
#define __NICODECIDS_H__

#include "XnTypes.h"

#define XN_CODEC_NULL				XN_CODEC_ID(0, 0, 0, 0)
#define XN_CODEC_UNCOMPRESSED		XN_CODEC_ID('N','O','N','E')
#define XN_CODEC_JPEG				XN_CODEC_ID('J','P','E','G')
#define XN_CODEC_16Z				XN_CODEC_ID('1','6','z','P')
#define XN_CODEC_16Z_EMB_TABLES		XN_CODEC_ID('1','6','z','T')
#define XN_CODEC_8Z					XN_CODEC_ID('I','m','8','z')

#endif // __NICODECIDS_H__
