/****************************************************************************
*                                                                           *
*  OpenNI 1.x Alpha                                                         *
*  Copyright (C) 2011 PrimeSense Ltd.                                       *
*                                                                           *
*  This file is part of OpenNI.                                             *
*                                                                           *
*  OpenNI is free software: you can redistribute it and/or modify           *
*  it under the terms of the GNU Lesser General Public License as published *
*  by the Free Software Foundation, either version 3 of the License, or     *
*  (at your option) any later version.                                      *
*                                                                           *
*  OpenNI is distributed in the hope that it will be useful,                *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of           *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             *
*  GNU Lesser General Public License for more details.                      *
*                                                                           *
*  You should have received a copy of the GNU Lesser General Public License *
*  along with OpenNI. If not, see <http://www.gnu.org/licenses/>.           *
*                                                                           *
****************************************************************************/
package org.OpenNI;

public enum Capability 
{
	ExtendedSerialization ("ExtendedSerialization"),
	Mirror ("Mirror"),
	AlternativeViewPoint ("AlternativeViewPoint"),
	Cropping ("Cropping"),
	UserPosition ("UserPosition"),
	Skeleton ("User::Skeleton"),
	PoseDetection ("User::PoseDetection"),
	LockAware ("LockAware"),
	ErrorState ("ErrorState"),
	FrameSync ("FrameSync"),
	DeviceIdentification ("DeviceIdentification"),
	Brightness ("Brightness"),
	Contrast ("Contrast"),
	Hue ("Hue"),
	Saturation ("Saturation"),
	Sharpness ("Sharpness"),
	Gamma ("Gamma"),
	WhiteBalance ("ColorTemperature"),
	BacklightCompensation ("BacklightCompensation"),
	Gain ("Gain"),
	Pan ("Pan"),
	Tilt ("Tilt"),
	Roll ("Roll"),
	Zoom ("Zoom"),
	Exposure ("Exposure"),
	Iris ("Iris"),
	Focus ("Focus"),
	LowLightCompensation ("LowLightCompensation"),
	AntiFlicker ("AntiFlicker");
	
	Capability(String name)
	{
		this.name = name;
	}
	
	public String getName() { return this.name; }
	
	private String name;
}
