#ifndef _JAVASOFT_JNI_MD_H_
#define _JAVASOFT_JNI_MD_H_

#include <XnPlatform.h>

#define JNIEXPORT XN_API_EXPORT
#define JNIIMPORT XN_API_IMPORT
#define JNICALL XN_CALLBACK_TYPE

typedef XnInt32 jint;
typedef XnInt64 jlong;
typedef XnInt8 jbyte;

#endif /* !_JAVASOFT_JNI_MD_H_ */
