/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class org_OpenNI_Samples_Assistant_NativeMethods */

#ifndef _Included_org_OpenNI_Samples_Assistant_NativeMethods
#define _Included_org_OpenNI_Samples_Assistant_NativeMethods
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     org_OpenNI_Samples_Assistant_NativeMethods
 * Method:    readByte
 * Signature: (J)B
 */
JNIEXPORT jbyte JNICALL Java_org_OpenNI_Samples_Assistant_NativeMethods_readByte
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_OpenNI_Samples_Assistant_NativeMethods
 * Method:    readShort
 * Signature: (J)S
 */
JNIEXPORT jshort JNICALL Java_org_OpenNI_Samples_Assistant_NativeMethods_readShort
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_OpenNI_Samples_Assistant_NativeMethods
 * Method:    readInt
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_org_OpenNI_Samples_Assistant_NativeMethods_readInt
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_OpenNI_Samples_Assistant_NativeMethods
 * Method:    readLong
 * Signature: (J)J
 */
JNIEXPORT jlong JNICALL Java_org_OpenNI_Samples_Assistant_NativeMethods_readLong
  (JNIEnv *, jclass, jlong);

/*
 * Class:     org_OpenNI_Samples_Assistant_NativeMethods
 * Method:    copyToBuffer
 * Signature: (Ljava/nio/ByteBuffer;JI)V
 */
JNIEXPORT void JNICALL Java_org_OpenNI_Samples_Assistant_NativeMethods_copyToBuffer
  (JNIEnv *, jclass, jobject, jlong, jint);

/*
 * Class:     org_OpenNI_Samples_Assistant_NativeMethods
 * Method:    initFromContext
 * Signature: (JZZ)I
 */
JNIEXPORT jint JNICALL Java_org_OpenNI_Samples_Assistant_NativeMethods_initFromContext
  (JNIEnv *, jclass, jlong, jboolean, jboolean);

/*
 * Class:     org_OpenNI_Samples_Assistant_NativeMethods
 * Method:    dispose
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_org_OpenNI_Samples_Assistant_NativeMethods_dispose
  (JNIEnv *, jclass);

/*
 * Class:     org_OpenNI_Samples_Assistant_NativeMethods
 * Method:    getMapOutputMode
 * Signature: (Lorg/OpenNI/OutArg;Lorg/OpenNI/OutArg;Lorg/OpenNI/OutArg;)I
 */
JNIEXPORT jint JNICALL Java_org_OpenNI_Samples_Assistant_NativeMethods_getMapOutputMode
  (JNIEnv *, jclass, jobject, jobject, jobject);

/*
 * Class:     org_OpenNI_Samples_Assistant_NativeMethods
 * Method:    generateBitmapLocalBuffer
 * Signature: (ZZZLorg/OpenNI/OutArg;)I
 */
JNIEXPORT jint JNICALL Java_org_OpenNI_Samples_Assistant_NativeMethods_generateBitmapLocalBuffer
  (JNIEnv *, jclass, jboolean, jboolean, jboolean, jobject);

/*
 * Class:     org_OpenNI_Samples_Assistant_NativeMethods
 * Method:    generateBitmapJavaBuffer
 * Signature: (ZZZ[I)I
 */
JNIEXPORT jint JNICALL Java_org_OpenNI_Samples_Assistant_NativeMethods_generateBitmapJavaBuffer
  (JNIEnv *, jclass, jboolean, jboolean, jboolean, jintArray);

/*
 * Class:     org_OpenNI_Samples_Assistant_NativeMethods
 * Method:    readLocalBitmap2JavaBuffer
 * Signature: (J[I)I
 */
JNIEXPORT jint JNICALL Java_org_OpenNI_Samples_Assistant_NativeMethods_readLocalBitmap2JavaBuffer
  (JNIEnv *, jclass, jlong, jintArray);

#ifdef __cplusplus
}
#endif
#endif
