@%0\..\..\..\BuildJava.py "%1" "%0\.." ..\..\..\..\..\Samples\HandTracker.java\org\OpenNI\Samples\HandTracker org.OpenNI.Samples.HandTracker org.OpenNI.jar org.OpenNI.Samples.HandTracker.HandTrackerApplication
