static JNINativeMethod methods[] = {
	{ "readByte", "(J)B", (void*)&Java_org_OpenNI_NativeMethods_readByte },
	{ "readShort", "(J)S", (void*)&Java_org_OpenNI_NativeMethods_readShort },
	{ "readInt", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_readInt },
	{ "readLong", "(J)J", (void*)&Java_org_OpenNI_NativeMethods_readLong },
	{ "copyToBuffer", "(Ljava/nio/ByteBuffer;JI)V", (void*)&Java_org_OpenNI_NativeMethods_copyToBuffer },
	{ "createProductionNodeDescription", "(ILjava/lang/String;Ljava/lang/String;BBSI)J", (void*)&Java_org_OpenNI_NativeMethods_createProductionNodeDescription },
	{ "freeProductionNodeDescription", "(J)V", (void*)&Java_org_OpenNI_NativeMethods_freeProductionNodeDescription },
	{ "xnGetStatusString", "(I)Ljava/lang/String;", (void*)&Java_org_OpenNI_NativeMethods_xnGetStatusString },
	{ "xnInit", "(Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnInit },
	{ "xnContextRunXmlScriptFromFileEx", "(JLjava/lang/String;JLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnContextRunXmlScriptFromFileEx },
	{ "xnContextRunXmlScriptEx", "(JLjava/lang/String;JLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnContextRunXmlScriptEx },
	{ "xnInitFromXmlFileEx", "(Ljava/lang/String;Lorg/OpenNI/OutArg;JLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnInitFromXmlFileEx },
	{ "xnContextOpenFileRecordingEx", "(JLjava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnContextOpenFileRecordingEx },
	{ "xnContextAddRef", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnContextAddRef },
	{ "xnContextRelease", "(J)V", (void*)&Java_org_OpenNI_NativeMethods_xnContextRelease },
	{ "xnEnumerateProductionTrees", "(JIJLorg/OpenNI/OutArg;J)I", (void*)&Java_org_OpenNI_NativeMethods_xnEnumerateProductionTrees },
	{ "xnCreateProductionTree", "(JJLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnCreateProductionTree },
	{ "xnCreateAnyProductionTree", "(JIJLorg/OpenNI/OutArg;J)I", (void*)&Java_org_OpenNI_NativeMethods_xnCreateAnyProductionTree },
	{ "xnEnumerateExistingNodes", "(JLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnEnumerateExistingNodes },
	{ "xnEnumerateExistingNodesByType", "(JILorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnEnumerateExistingNodesByType },
	{ "xnFindExistingRefNodeByType", "(JILorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnFindExistingRefNodeByType },
	{ "xnGetRefNodeHandleByName", "(JLjava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetRefNodeHandleByName },
	{ "xnWaitAndUpdateAll", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnWaitAndUpdateAll },
	{ "xnWaitOneUpdateAll", "(JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnWaitOneUpdateAll },
	{ "xnWaitAnyUpdateAll", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnWaitAnyUpdateAll },
	{ "xnWaitNoneUpdateAll", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnWaitNoneUpdateAll },
	{ "xnStartGeneratingAll", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnStartGeneratingAll },
	{ "xnStopGeneratingAll", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnStopGeneratingAll },
	{ "xnSetGlobalMirror", "(JZ)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetGlobalMirror },
	{ "xnGetGlobalMirror", "(J)Z", (void*)&Java_org_OpenNI_NativeMethods_xnGetGlobalMirror },
	{ "xnGetGlobalErrorState", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetGlobalErrorState },
	{ "xnRegisterToGlobalErrorStateChange", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToGlobalErrorStateChange },
	{ "xnUnregisterFromGlobalErrorStateChange", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromGlobalErrorStateChange },
	{ "xnRegisterToNodeCreation", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToNodeCreation },
	{ "xnUnregisterFromNodeCreation", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromNodeCreation },
	{ "xnRegisterToNodeDestruction", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToNodeDestruction },
	{ "xnUnregisterFromNodeDestruction", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromNodeDestruction },
	{ "xnAddLicense", "(JLjava/lang/String;Ljava/lang/String;)I", (void*)&Java_org_OpenNI_NativeMethods_xnAddLicense },
	{ "xnEnumerateLicenses", "(JLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnEnumerateLicenses },
	{ "xnEnumerationErrorsAllocate", "(Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnEnumerationErrorsAllocate },
	{ "xnEnumerationErrorsFree", "(J)V", (void*)&Java_org_OpenNI_NativeMethods_xnEnumerationErrorsFree },
	{ "xnEnumerationErrorsToString", "(JLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnEnumerationErrorsToString },
	{ "xnEnumerationErrorsClear", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnEnumerationErrorsClear },
	{ "xnEnumerationErrorsGetFirst", "(J)J", (void*)&Java_org_OpenNI_NativeMethods_xnEnumerationErrorsGetFirst },
	{ "xnEnumerationErrorsIteratorIsValid", "(J)Z", (void*)&Java_org_OpenNI_NativeMethods_xnEnumerationErrorsIteratorIsValid },
	{ "xnNodeInfoSetInstanceName", "(JLjava/lang/String;)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeInfoSetInstanceName },
	{ "xnNodeInfoGetDescription", "(J)Lorg/OpenNI/ProductionNodeDescription;", (void*)&Java_org_OpenNI_NativeMethods_xnNodeInfoGetDescription },
	{ "xnNodeInfoGetTreeStringRepresentation", "(JLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeInfoGetTreeStringRepresentation },
	{ "xnNodeInfoGetInstanceName", "(J)Ljava/lang/String;", (void*)&Java_org_OpenNI_NativeMethods_xnNodeInfoGetInstanceName },
	{ "xnNodeInfoGetCreationInfo", "(J)Ljava/lang/String;", (void*)&Java_org_OpenNI_NativeMethods_xnNodeInfoGetCreationInfo },
	{ "xnNodeInfoGetNeededNodes", "(J)J", (void*)&Java_org_OpenNI_NativeMethods_xnNodeInfoGetNeededNodes },
	{ "xnNodeInfoGetRefHandle", "(J)J", (void*)&Java_org_OpenNI_NativeMethods_xnNodeInfoGetRefHandle },
	{ "xnNodeInfoListAllocate", "(Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeInfoListAllocate },
	{ "xnNodeInfoListFree", "(J)V", (void*)&Java_org_OpenNI_NativeMethods_xnNodeInfoListFree },
	{ "xnNodeInfoListAdd", "(JJLjava/lang/String;J)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeInfoListAdd },
	{ "xnNodeInfoListAddNode", "(JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeInfoListAddNode },
	{ "xnNodeInfoListAddNodeFromList", "(JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeInfoListAddNodeFromList },
	{ "xnNodeInfoListRemove", "(JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeInfoListRemove },
	{ "xnNodeInfoListClear", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeInfoListClear },
	{ "xnNodeInfoListAppend", "(JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeInfoListAppend },
	{ "xnNodeInfoListIsEmpty", "(J)Z", (void*)&Java_org_OpenNI_NativeMethods_xnNodeInfoListIsEmpty },
	{ "xnNodeInfoListGetFirst", "(J)J", (void*)&Java_org_OpenNI_NativeMethods_xnNodeInfoListGetFirst },
	{ "xnNodeInfoListIteratorIsValid", "(J)Z", (void*)&Java_org_OpenNI_NativeMethods_xnNodeInfoListIteratorIsValid },
	{ "xnNodeInfoListGetCurrent", "(J)J", (void*)&Java_org_OpenNI_NativeMethods_xnNodeInfoListGetCurrent },
	{ "xnNodeInfoListGetNext", "(J)J", (void*)&Java_org_OpenNI_NativeMethods_xnNodeInfoListGetNext },
	{ "xnNodeQueryAllocate", "(Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeQueryAllocate },
	{ "xnNodeQueryFree", "(J)V", (void*)&Java_org_OpenNI_NativeMethods_xnNodeQueryFree },
	{ "xnNodeQuerySetVendor", "(JLjava/lang/String;)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeQuerySetVendor },
	{ "xnNodeQuerySetName", "(JLjava/lang/String;)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeQuerySetName },
	{ "xnNodeQuerySetMinVersion", "(JBBSI)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeQuerySetMinVersion },
	{ "xnNodeQuerySetMaxVersion", "(JBBSI)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeQuerySetMaxVersion },
	{ "xnNodeQueryAddSupportedCapability", "(JLjava/lang/String;)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeQueryAddSupportedCapability },
	{ "xnNodeQueryAddSupportedMapOutputMode", "(JIII)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeQueryAddSupportedMapOutputMode },
	{ "xnNodeQuerySetSupportedMinUserPositions", "(JI)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeQuerySetSupportedMinUserPositions },
	{ "xnNodeQuerySetExistingNodeOnly", "(JZ)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeQuerySetExistingNodeOnly },
	{ "xnNodeQuerySetNonExistingNodeOnly", "(JZ)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeQuerySetNonExistingNodeOnly },
	{ "xnNodeQueryAddNeededNode", "(JLjava/lang/String;)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeQueryAddNeededNode },
	{ "xnNodeQuerySetCreationInfo", "(JLjava/lang/String;)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeQuerySetCreationInfo },
	{ "xnNodeQueryFilterList", "(JJJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnNodeQueryFilterList },
	{ "xnProductionNodeAddRef", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnProductionNodeAddRef },
	{ "xnProductionNodeRelease", "(J)V", (void*)&Java_org_OpenNI_NativeMethods_xnProductionNodeRelease },
	{ "xnGetNodeInfo", "(J)J", (void*)&Java_org_OpenNI_NativeMethods_xnGetNodeInfo },
	{ "xnGetNodeName", "(J)Ljava/lang/String;", (void*)&Java_org_OpenNI_NativeMethods_xnGetNodeName },
	{ "xnGetRefContextFromNodeHandle", "(J)J", (void*)&Java_org_OpenNI_NativeMethods_xnGetRefContextFromNodeHandle },
	{ "xnIsCapabilitySupported", "(JLjava/lang/String;)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsCapabilitySupported },
	{ "xnSetIntProperty", "(JLjava/lang/String;J)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetIntProperty },
	{ "xnSetRealProperty", "(JLjava/lang/String;D)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetRealProperty },
	{ "xnSetStringProperty", "(JLjava/lang/String;Ljava/lang/String;)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetStringProperty },
	{ "xnSetGeneralProperty", "(JLjava/lang/String;IJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetGeneralProperty },
	{ "xnSetGeneralPropertyArray", "(JLjava/lang/String;[B)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetGeneralPropertyArray },
	{ "xnGetIntProperty", "(JLjava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetIntProperty },
	{ "xnGetRealProperty", "(JLjava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetRealProperty },
	{ "xnGetStringProperty", "(JLjava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetStringProperty },
	{ "xnGetGeneralProperty", "(JLjava/lang/String;IJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetGeneralProperty },
	{ "xnGetGeneralPropertyArray", "(JLjava/lang/String;[B)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetGeneralPropertyArray },
	{ "xnLockNodeForChanges", "(JLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnLockNodeForChanges },
	{ "xnUnlockNodeForChanges", "(JI)I", (void*)&Java_org_OpenNI_NativeMethods_xnUnlockNodeForChanges },
	{ "xnLockedNodeStartChanges", "(JI)I", (void*)&Java_org_OpenNI_NativeMethods_xnLockedNodeStartChanges },
	{ "xnLockedNodeEndChanges", "(JI)I", (void*)&Java_org_OpenNI_NativeMethods_xnLockedNodeEndChanges },
	{ "xnAddNeededNode", "(JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnAddNeededNode },
	{ "xnRemoveNeededNode", "(JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnRemoveNeededNode },
	{ "xnCreateDevice", "(JLorg/OpenNI/OutArg;JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnCreateDevice },
	{ "xnGetDeviceName", "(JLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetDeviceName },
	{ "xnGetVendorSpecificData", "(JLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetVendorSpecificData },
	{ "xnGetSerialNumber", "(JLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetSerialNumber },
	{ "xnGetNodeErrorState", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetNodeErrorState },
	{ "xnRegisterToNodeErrorStateChange", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToNodeErrorStateChange },
	{ "xnUnregisterFromNodeErrorStateChange", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromNodeErrorStateChange },
	{ "xnGetGeneralIntRange", "(JLjava/lang/String;Lorg/OpenNI/OutArg;Lorg/OpenNI/OutArg;Lorg/OpenNI/OutArg;Lorg/OpenNI/OutArg;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetGeneralIntRange },
	{ "xnGetGeneralIntValue", "(JLjava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetGeneralIntValue },
	{ "xnSetGeneralIntValue", "(JLjava/lang/String;I)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetGeneralIntValue },
	{ "xnRegisterToGeneralIntValueChange", "(JLjava/lang/String;Ljava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToGeneralIntValueChange },
	{ "xnUnregisterFromGeneralIntValueChange", "(JLjava/lang/String;J)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromGeneralIntValueChange },
	{ "xnStartGenerating", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnStartGenerating },
	{ "xnIsGenerating", "(J)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsGenerating },
	{ "xnStopGenerating", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnStopGenerating },
	{ "xnRegisterToGenerationRunningChange", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToGenerationRunningChange },
	{ "xnUnregisterFromGenerationRunningChange", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromGenerationRunningChange },
	{ "xnRegisterToNewDataAvailable", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToNewDataAvailable },
	{ "xnUnregisterFromNewDataAvailable", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromNewDataAvailable },
	{ "xnIsNewDataAvailable", "(JLorg/OpenNI/OutArg;)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsNewDataAvailable },
	{ "xnWaitAndUpdateData", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnWaitAndUpdateData },
	{ "xnIsDataNew", "(J)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsDataNew },
	{ "xnGetData", "(J)J", (void*)&Java_org_OpenNI_NativeMethods_xnGetData },
	{ "xnGetDataSize", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetDataSize },
	{ "xnGetTimestamp", "(J)J", (void*)&Java_org_OpenNI_NativeMethods_xnGetTimestamp },
	{ "xnGetFrameID", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetFrameID },
	{ "xnSetMirror", "(JZ)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetMirror },
	{ "xnIsMirrored", "(J)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsMirrored },
	{ "xnRegisterToMirrorChange", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToMirrorChange },
	{ "xnUnregisterFromMirrorChange", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromMirrorChange },
	{ "xnIsViewPointSupported", "(JJ)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsViewPointSupported },
	{ "xnSetViewPoint", "(JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetViewPoint },
	{ "xnResetViewPoint", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnResetViewPoint },
	{ "xnIsViewPointAs", "(JJ)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsViewPointAs },
	{ "xnRegisterToViewPointChange", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToViewPointChange },
	{ "xnUnregisterFromViewPointChange", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromViewPointChange },
	{ "xnCanFrameSyncWith", "(JJ)Z", (void*)&Java_org_OpenNI_NativeMethods_xnCanFrameSyncWith },
	{ "xnFrameSyncWith", "(JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnFrameSyncWith },
	{ "xnStopFrameSyncWith", "(JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnStopFrameSyncWith },
	{ "xnIsFrameSyncedWith", "(JJ)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsFrameSyncedWith },
	{ "xnRegisterToFrameSyncChange", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToFrameSyncChange },
	{ "xnUnregisterFromFrameSyncChange", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromFrameSyncChange },
	{ "xnGetSupportedMapOutputModesCount", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetSupportedMapOutputModesCount },
	{ "xnGetSupportedMapOutputModes", "(J[Lorg/OpenNI/MapOutputMode;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetSupportedMapOutputModes },
	{ "xnSetMapOutputMode", "(JIII)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetMapOutputMode },
	{ "xnGetMapOutputMode", "(JLorg/OpenNI/OutArg;Lorg/OpenNI/OutArg;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetMapOutputMode },
	{ "xnRegisterToMapOutputModeChange", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToMapOutputModeChange },
	{ "xnUnregisterFromMapOutputModeChange", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromMapOutputModeChange },
	{ "xnGetBytesPerPixel", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetBytesPerPixel },
	{ "xnSetCropping", "(JIIIIZ)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetCropping },
	{ "xnGetCropping", "(JLorg/OpenNI/OutArg;Lorg/OpenNI/OutArg;Lorg/OpenNI/OutArg;Lorg/OpenNI/OutArg;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetCropping },
	{ "xnRegisterToCroppingChange", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToCroppingChange },
	{ "xnUnregisterFromCroppingChange", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromCroppingChange },
	{ "xnSetPowerLineFrequency", "(JI)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetPowerLineFrequency },
	{ "xnGetPowerLineFrequency", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetPowerLineFrequency },
	{ "xnRegisterToPowerLineFrequencyChange", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToPowerLineFrequencyChange },
	{ "xnUnregisterFromPowerLineFrequencyChange", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromPowerLineFrequencyChange },
	{ "xnCreateDepthGenerator", "(JLorg/OpenNI/OutArg;JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnCreateDepthGenerator },
	{ "xnGetDeviceMaxDepth", "(J)S", (void*)&Java_org_OpenNI_NativeMethods_xnGetDeviceMaxDepth },
	{ "xnGetDepthFieldOfView", "(JLorg/OpenNI/OutArg;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetDepthFieldOfView },
	{ "xnRegisterToDepthFieldOfViewChange", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToDepthFieldOfViewChange },
	{ "xnUnregisterFromDepthFieldOfViewChange", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromDepthFieldOfViewChange },
	{ "xnConvertProjectiveToRealWorld", "(J[Lorg/OpenNI/Point3D;[Lorg/OpenNI/Point3D;)I", (void*)&Java_org_OpenNI_NativeMethods_xnConvertProjectiveToRealWorld },
	{ "xnConvertRealWorldToProjective", "(J[Lorg/OpenNI/Point3D;[Lorg/OpenNI/Point3D;)I", (void*)&Java_org_OpenNI_NativeMethods_xnConvertRealWorldToProjective },
	{ "xnGetDepthMap", "(J)J", (void*)&Java_org_OpenNI_NativeMethods_xnGetDepthMap },
	{ "xnGetDepthMetaData", "(JLorg/OpenNI/DepthMetaData;)V", (void*)&Java_org_OpenNI_NativeMethods_xnGetDepthMetaData },
	{ "xnGetSupportedUserPositionsCount", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetSupportedUserPositionsCount },
	{ "xnSetUserPosition", "(JIFFFFFF)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetUserPosition },
	{ "xnGetUserPosition", "(JILorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetUserPosition },
	{ "xnRegisterToUserPositionChange", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToUserPositionChange },
	{ "xnUnregisterFromUserPositionChange", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromUserPositionChange },
	{ "xnCreateImageGenerator", "(JLorg/OpenNI/OutArg;JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnCreateImageGenerator },
	{ "xnGetImageMap", "(J)J", (void*)&Java_org_OpenNI_NativeMethods_xnGetImageMap },
	{ "xnIsPixelFormatSupported", "(JI)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsPixelFormatSupported },
	{ "xnSetPixelFormat", "(JI)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetPixelFormat },
	{ "xnGetPixelFormat", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetPixelFormat },
	{ "xnRegisterToPixelFormatChange", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToPixelFormatChange },
	{ "xnUnregisterFromPixelFormatChange", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromPixelFormatChange },
	{ "xnGetImageMetaData", "(JLorg/OpenNI/ImageMetaData;)V", (void*)&Java_org_OpenNI_NativeMethods_xnGetImageMetaData },
	{ "xnCreateIRGenerator", "(JLorg/OpenNI/OutArg;JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnCreateIRGenerator },
	{ "xnGetIRMap", "(J)J", (void*)&Java_org_OpenNI_NativeMethods_xnGetIRMap },
	{ "xnGetIRMetaData", "(JLorg/OpenNI/IRMetaData;)V", (void*)&Java_org_OpenNI_NativeMethods_xnGetIRMetaData },
	{ "xnCreateGestureGenerator", "(JLorg/OpenNI/OutArg;JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnCreateGestureGenerator },
	{ "xnAddGesture", "(JLjava/lang/String;)I", (void*)&Java_org_OpenNI_NativeMethods_xnAddGesture__JLjava_lang_String_2 },
	{ "xnAddGesture", "(JLjava/lang/String;FFFFFF)I", (void*)&Java_org_OpenNI_NativeMethods_xnAddGesture__JLjava_lang_String_2FFFFFF },
	{ "xnRemoveGesture", "(JLjava/lang/String;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRemoveGesture },
	{ "xnGetAllActiveGestures", "(JLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetAllActiveGestures },
	{ "xnGetNumberOfAvailableGestures", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetNumberOfAvailableGestures },
	{ "xnEnumerateAllGestures", "(JLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnEnumerateAllGestures },
	{ "xnIsGestureAvailable", "(JLjava/lang/String;)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsGestureAvailable },
	{ "xnIsGestureProgressSupported", "(JLjava/lang/String;)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsGestureProgressSupported },
	{ "xnRegisterGestureCallbacks", "(JLjava/lang/Object;Ljava/lang/String;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterGestureCallbacks },
	{ "xnUnregisterGestureCallbacks", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterGestureCallbacks },
	{ "xnRegisterToGestureChange", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToGestureChange },
	{ "xnUnregisterFromGestureChange", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromGestureChange },
	{ "xnRegisterToGestureIntermediateStageCompleted", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToGestureIntermediateStageCompleted },
	{ "xnUnregisterFromGestureIntermediateStageCompleted", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromGestureIntermediateStageCompleted },
	{ "xnRegisterToGestureReadyForNextIntermediateStage", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToGestureReadyForNextIntermediateStage },
	{ "xnUnregisterFromGestureReadyForNextIntermediateStage", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromGestureReadyForNextIntermediateStage },
	{ "xnCreateSceneAnalyzer", "(JLorg/OpenNI/OutArg;JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnCreateSceneAnalyzer },
	{ "xnGetLabelMap", "(J)J", (void*)&Java_org_OpenNI_NativeMethods_xnGetLabelMap },
	{ "xnGetFloor", "(JLorg/OpenNI/OutArg;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetFloor },
	{ "xnGetSceneMetaData", "(JLorg/OpenNI/SceneMetaData;)V", (void*)&Java_org_OpenNI_NativeMethods_xnGetSceneMetaData },
	{ "xnCreateUserGenerator", "(JLorg/OpenNI/OutArg;JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnCreateUserGenerator },
	{ "xnGetNumberOfUsers", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetNumberOfUsers },
	{ "xnGetUsers", "(JLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetUsers },
	{ "xnGetUserCoM", "(JILorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetUserCoM },
	{ "xnGetUserPixels", "(JILorg/OpenNI/SceneMetaData;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetUserPixels },
	{ "xnRegisterUserCallbacks", "(JLjava/lang/Object;Ljava/lang/String;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterUserCallbacks },
	{ "xnUnregisterUserCallbacks", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterUserCallbacks },
	{ "xnRegisterToUserExit", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToUserExit },
	{ "xnUnregisterFromUserExit", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromUserExit },
	{ "xnRegisterToUserReEnter", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToUserReEnter },
	{ "xnUnregisterFromUserReEnter", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromUserReEnter },
	{ "xnIsJointAvailable", "(JI)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsJointAvailable },
	{ "xnIsProfileAvailable", "(JI)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsProfileAvailable },
	{ "xnSetSkeletonProfile", "(JI)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetSkeletonProfile },
	{ "xnSetJointActive", "(JIZ)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetJointActive },
	{ "xnIsJointActive", "(JI)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsJointActive },
	{ "xnRegisterToJointConfigurationChange", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToJointConfigurationChange },
	{ "xnUnregisterFromJointConfigurationChange", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromJointConfigurationChange },
	{ "xnEnumerateActiveJoints", "(JLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnEnumerateActiveJoints },
	{ "xnGetSkeletonJoint", "(JIILorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetSkeletonJoint },
	{ "xnGetSkeletonJointPosition", "(JIILorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetSkeletonJointPosition },
	{ "xnGetSkeletonJointOrientation", "(JIILorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetSkeletonJointOrientation },
	{ "xnIsSkeletonTracking", "(JI)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsSkeletonTracking },
	{ "xnIsSkeletonCalibrated", "(JI)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsSkeletonCalibrated },
	{ "xnIsSkeletonCalibrating", "(JI)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsSkeletonCalibrating },
	{ "xnRequestSkeletonCalibration", "(JIZ)I", (void*)&Java_org_OpenNI_NativeMethods_xnRequestSkeletonCalibration },
	{ "xnAbortSkeletonCalibration", "(JI)I", (void*)&Java_org_OpenNI_NativeMethods_xnAbortSkeletonCalibration },
	{ "xnSaveSkeletonCalibrationDataToFile", "(JILjava/lang/String;)I", (void*)&Java_org_OpenNI_NativeMethods_xnSaveSkeletonCalibrationDataToFile },
	{ "xnLoadSkeletonCalibrationDataFromFile", "(JILjava/lang/String;)I", (void*)&Java_org_OpenNI_NativeMethods_xnLoadSkeletonCalibrationDataFromFile },
	{ "xnSaveSkeletonCalibrationData", "(JII)I", (void*)&Java_org_OpenNI_NativeMethods_xnSaveSkeletonCalibrationData },
	{ "xnLoadSkeletonCalibrationData", "(JII)I", (void*)&Java_org_OpenNI_NativeMethods_xnLoadSkeletonCalibrationData },
	{ "xnClearSkeletonCalibrationData", "(JI)I", (void*)&Java_org_OpenNI_NativeMethods_xnClearSkeletonCalibrationData },
	{ "xnIsSkeletonCalibrationData", "(JI)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsSkeletonCalibrationData },
	{ "xnStartSkeletonTracking", "(JI)I", (void*)&Java_org_OpenNI_NativeMethods_xnStartSkeletonTracking },
	{ "xnStopSkeletonTracking", "(JI)I", (void*)&Java_org_OpenNI_NativeMethods_xnStopSkeletonTracking },
	{ "xnResetSkeleton", "(JI)I", (void*)&Java_org_OpenNI_NativeMethods_xnResetSkeleton },
	{ "xnNeedPoseForSkeletonCalibration", "(J)Z", (void*)&Java_org_OpenNI_NativeMethods_xnNeedPoseForSkeletonCalibration },
	{ "xnGetSkeletonCalibrationPose", "(JLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetSkeletonCalibrationPose },
	{ "xnSetSkeletonSmoothing", "(JF)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetSkeletonSmoothing },
	{ "xnRegisterToCalibrationStart", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToCalibrationStart },
	{ "xnUnregisterFromCalibrationStart", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromCalibrationStart },
	{ "xnRegisterToCalibrationInProgress", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToCalibrationInProgress },
	{ "xnUnregisterFromCalibrationInProgress", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromCalibrationInProgress },
	{ "xnRegisterToCalibrationComplete", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToCalibrationComplete },
	{ "xnUnregisterFromCalibrationComplete", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromCalibrationComplete },
	{ "xnGetNumberOfPoses", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetNumberOfPoses },
	{ "xnGetAllAvailablePoses", "(JLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetAllAvailablePoses },
	{ "xnStartPoseDetection", "(JLjava/lang/String;I)I", (void*)&Java_org_OpenNI_NativeMethods_xnStartPoseDetection },
	{ "xnStopPoseDetection", "(JI)I", (void*)&Java_org_OpenNI_NativeMethods_xnStopPoseDetection },
	{ "xnStopSinglePoseDetection", "(JILjava/lang/String;)I", (void*)&Java_org_OpenNI_NativeMethods_xnStopSinglePoseDetection },
	{ "xnRegisterToPoseDetected", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToPoseDetected },
	{ "xnUnregisterFromPoseDetected", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromPoseDetected },
	{ "xnRegisterToOutOfPose", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToOutOfPose },
	{ "xnUnregisterFromOutOfPose", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromOutOfPose },
	{ "xnRegisterToPoseDetectionInProgress", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToPoseDetectionInProgress },
	{ "xnUnregisterFromPoseDetectionInProgress", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromPoseDetectionInProgress },
	{ "xnIsPoseSupported", "(JLjava/lang/String;)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsPoseSupported },
	{ "xnGetPoseStatus", "(JILjava/lang/String;Lorg/OpenNI/OutArg;Lorg/OpenNI/OutArg;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetPoseStatus },
	{ "xnCreateHandsGenerator", "(JLorg/OpenNI/OutArg;JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnCreateHandsGenerator },
	{ "xnRegisterHandCallbacks", "(JLjava/lang/Object;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterHandCallbacks },
	{ "xnUnregisterHandCallbacks", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterHandCallbacks },
	{ "xnStopTracking", "(JI)I", (void*)&Java_org_OpenNI_NativeMethods_xnStopTracking },
	{ "xnStopTrackingAll", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnStopTrackingAll },
	{ "xnStartTracking", "(JFFF)I", (void*)&Java_org_OpenNI_NativeMethods_xnStartTracking },
	{ "xnSetTrackingSmoothing", "(JF)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetTrackingSmoothing },
	{ "xnRegisterToHandTouchingFOVEdge", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToHandTouchingFOVEdge },
	{ "xnUnregisterFromHandTouchingFOVEdge", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromHandTouchingFOVEdge },
	{ "xnCreateAudioGenerator", "(JLorg/OpenNI/OutArg;JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnCreateAudioGenerator },
	{ "xnGetAudioBuffer", "(J)J", (void*)&Java_org_OpenNI_NativeMethods_xnGetAudioBuffer },
	{ "xnGetSupportedWaveOutputModesCount", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetSupportedWaveOutputModesCount },
	{ "xnGetSupportedWaveOutputModes", "(J[Lorg/OpenNI/WaveOutputMode;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetSupportedWaveOutputModes },
	{ "xnSetWaveOutputMode", "(JISB)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetWaveOutputMode },
	{ "xnGetWaveOutputMode", "(JLorg/OpenNI/OutArg;Lorg/OpenNI/OutArg;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetWaveOutputMode },
	{ "xnRegisterToWaveOutputModeChanges", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToWaveOutputModeChanges },
	{ "xnUnregisterFromWaveOutputModeChanges", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromWaveOutputModeChanges },
	{ "xnGetAudioMetaData", "(JLorg/OpenNI/AudioMetaData;)V", (void*)&Java_org_OpenNI_NativeMethods_xnGetAudioMetaData },
	{ "xnCreateCodec", "(JIJLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnCreateCodec },
	{ "xnGetCodecID", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetCodecID },
	{ "xnEncodeData", "(JJIJILorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnEncodeData },
	{ "xnDecodeData", "(JJIJILorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnDecodeData },
	{ "xnCreateRecorder", "(JLjava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnCreateRecorder },
	{ "xnSetRecorderDestination", "(JILjava/lang/String;)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetRecorderDestination },
	{ "xnGetRecorderDestination", "(JLorg/OpenNI/OutArg;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetRecorderDestination },
	{ "xnAddNodeToRecording", "(JJI)I", (void*)&Java_org_OpenNI_NativeMethods_xnAddNodeToRecording },
	{ "xnRemoveNodeFromRecording", "(JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnRemoveNodeFromRecording },
	{ "xnRecord", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnRecord },
	{ "xnGetRecorderFormat", "(J)Ljava/lang/String;", (void*)&Java_org_OpenNI_NativeMethods_xnGetRecorderFormat },
	{ "xnCreatePlayer", "(JLjava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnCreatePlayer },
	{ "xnSetPlayerRepeat", "(JZ)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetPlayerRepeat },
	{ "xnSetPlayerSource", "(JILjava/lang/String;)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetPlayerSource },
	{ "xnGetPlayerSource", "(JLorg/OpenNI/OutArg;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetPlayerSource },
	{ "xnPlayerReadNext", "(J)I", (void*)&Java_org_OpenNI_NativeMethods_xnPlayerReadNext },
	{ "xnSeekPlayerToTimeStamp", "(JJI)I", (void*)&Java_org_OpenNI_NativeMethods_xnSeekPlayerToTimeStamp },
	{ "xnSeekPlayerToFrame", "(JLjava/lang/String;II)I", (void*)&Java_org_OpenNI_NativeMethods_xnSeekPlayerToFrame },
	{ "xnTellPlayerTimestamp", "(JLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnTellPlayerTimestamp },
	{ "xnTellPlayerFrame", "(JLjava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnTellPlayerFrame },
	{ "xnGetPlayerNumFrames", "(JLjava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetPlayerNumFrames },
	{ "xnGetPlayerSupportedFormat", "(J)Ljava/lang/String;", (void*)&Java_org_OpenNI_NativeMethods_xnGetPlayerSupportedFormat },
	{ "xnEnumeratePlayerNodes", "(JLorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnEnumeratePlayerNodes },
	{ "xnIsPlayerAtEOF", "(J)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsPlayerAtEOF },
	{ "xnRegisterToEndOfFileReached", "(JLjava/lang/Object;Ljava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnRegisterToEndOfFileReached },
	{ "xnUnregisterFromEndOfFileReached", "(JJ)V", (void*)&Java_org_OpenNI_NativeMethods_xnUnregisterFromEndOfFileReached },
	{ "xnSetPlaybackSpeed", "(JD)I", (void*)&Java_org_OpenNI_NativeMethods_xnSetPlaybackSpeed },
	{ "xnGetPlaybackSpeed", "(J)D", (void*)&Java_org_OpenNI_NativeMethods_xnGetPlaybackSpeed },
	{ "xnCreateScriptNode", "(JLjava/lang/String;Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnCreateScriptNode },
	{ "xnScriptNodeGetSupportedFormat", "(J)Ljava/lang/String;", (void*)&Java_org_OpenNI_NativeMethods_xnScriptNodeGetSupportedFormat },
	{ "xnLoadScriptFromFile", "(JLjava/lang/String;)I", (void*)&Java_org_OpenNI_NativeMethods_xnLoadScriptFromFile },
	{ "xnLoadScriptFromString", "(JLjava/lang/String;)I", (void*)&Java_org_OpenNI_NativeMethods_xnLoadScriptFromString },
	{ "xnScriptNodeRun", "(JJ)I", (void*)&Java_org_OpenNI_NativeMethods_xnScriptNodeRun },
	{ "xnProductionNodeTypeToString", "(I)Ljava/lang/String;", (void*)&Java_org_OpenNI_NativeMethods_xnProductionNodeTypeToString },
	{ "xnResolutionGetXRes", "(I)I", (void*)&Java_org_OpenNI_NativeMethods_xnResolutionGetXRes },
	{ "xnResolutionGetYRes", "(I)I", (void*)&Java_org_OpenNI_NativeMethods_xnResolutionGetYRes },
	{ "xnResolutionGetFromXYRes", "(II)I", (void*)&Java_org_OpenNI_NativeMethods_xnResolutionGetFromXYRes },
	{ "xnResolutionGetFromName", "(Ljava/lang/String;)I", (void*)&Java_org_OpenNI_NativeMethods_xnResolutionGetFromName },
	{ "xnResolutionGetName", "(I)Ljava/lang/String;", (void*)&Java_org_OpenNI_NativeMethods_xnResolutionGetName },
	{ "xnGetVersion", "(Lorg/OpenNI/OutArg;)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetVersion },
	{ "xnIsTypeGenerator", "(I)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsTypeGenerator },
	{ "xnIsTypeDerivedFrom", "(II)Z", (void*)&Java_org_OpenNI_NativeMethods_xnIsTypeDerivedFrom },
	{ "xnGetBytesPerPixelForPixelFormat", "(I)I", (void*)&Java_org_OpenNI_NativeMethods_xnGetBytesPerPixelForPixelFormat },
};
